package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.busi.jx;

import cn.estsh.i3plus.ext.mes.api.busi.jx.IJxCenterWorkingBoardService;
import cn.estsh.i3plus.ext.mes.pojo.bean.MesFourMDetail;
import cn.estsh.i3plus.ext.mes.pojo.bean.MesShiftProdCenterRecord;
import cn.estsh.i3plus.ext.mes.pojo.bean.MesWorkCenterExt;
import cn.estsh.i3plus.ext.mes.pojo.bean.MesWorkCenterNotificationBar;
import cn.estsh.i3plus.ext.mes.pojo.repository.MesFourMDetailRepository;
import cn.estsh.i3plus.ext.mes.pojo.repository.MesWorkCenterNotificationBarRepository;
import cn.estsh.i3plus.ext.mes.pojo.util.MesExtConstWords;
import cn.estsh.i3plus.ext.mes.pojo.util.MesExtEnumUtil;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.impp.framework.boot.util.SpringContextsUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.util.CollectionUtils;

import java.util.Comparator;
import java.util.List;
import java.util.Map;
import java.util.StringJoiner;
import java.util.stream.Collectors;

/**
 * @Author : wangjie
 * @CreateDate : 2022-02-15
 * @Modify:
 **/
@Slf4j
public class JxCenterWorkingBoardBusiOneService implements IJxCenterWorkingBoardService {

    private IJxCenterWorkingBoardService proxyService;

    private MesFourMDetailRepository fourMDetailRepository;

    private MesWorkCenterNotificationBarRepository workCenterNotificationBarRepository;

    public JxCenterWorkingBoardBusiOneService(IJxCenterWorkingBoardService proxyService) {
        this.proxyService = proxyService;
        this.fourMDetailRepository = (MesFourMDetailRepository) SpringContextsUtil.getBean("mesFourMDetailRepository");
        this.workCenterNotificationBarRepository = (MesWorkCenterNotificationBarRepository) SpringContextsUtil.getBean("mesWorkCenterNotificationBarRepository");
    }

    @Override
    public Map<String, Object> queryCenterWorkingBoardData(Map<String, Object> dataMap) {

        dataMap = proxyService.queryCenterWorkingBoardData(dataMap);

        MesWorkCenterExt workCenterExtDb = (MesWorkCenterExt) dataMap.get(MesWorkCenterExt.class.getSimpleName());

        String startTime = (String) dataMap.get(MesExtConstWords.START_TIME);

        List<MesShiftProdCenterRecord> shiftProdCenterRecordList = (List<MesShiftProdCenterRecord>) dataMap.get(MesShiftProdCenterRecord.class.getSimpleName());

        dataMap.put("生产状态", getWorkingStatusData(shiftProdCenterRecordList, startTime));

        dataMap.put("异常停工时间", getUnnormalStopCenterData(shiftProdCenterRecordList, startTime));

        dataMap.put("4M变更信息", getFourMChangeData(workCenterExtDb, startTime));

        dataMap.put("公告信息", getWorkCenterNotificationBarData(workCenterExtDb));

        return clearUselessData(dataMap);
    }

    @Override
    public Map<String, Object> clearUselessData(Map<String, Object> dataMap) {
        dataMap.remove(MesWorkCenterExt.class.getSimpleName());
        dataMap.remove(MesShiftProdCenterRecord.class.getSimpleName());
        return dataMap;
    }

    private String getWorkingStatusData(List<MesShiftProdCenterRecord> shiftProdCenterRecordList, String startTime) {

        List<MesShiftProdCenterRecord> filterList = CollectionUtils.isEmpty(shiftProdCenterRecordList) ? null :
                shiftProdCenterRecordList.stream().filter(o -> (null != o && MesExtEnumUtil.SHIFT_PROD_TYPE.UNNORMAL_PROD.getValue() >= o.getProdType() && o.getStartTime().compareTo(startTime) <= 0 && o.getEndTime().compareTo(startTime) >= 0)).collect(Collectors.toList());
        MesShiftProdCenterRecord filterData = CollectionUtils.isEmpty(filterList) ? null :
                filterList.stream().filter(o -> null != o).sorted(Comparator.comparing(MesShiftProdCenterRecord::getModifyDatetime).reversed()).collect(Collectors.toList()).get(0);

        SHIFT_PROD_TYPE_COLOR enumItem;
        if (null == filterData) enumItem = SHIFT_PROD_TYPE_COLOR.NO_WORKING;
        else enumItem = SHIFT_PROD_TYPE_COLOR.getByValue(filterData.getProdType());
        return new StringJoiner(MesExtConstWords.COMMA).add(enumItem.workingStatusName).add(enumItem.color).toString();
    }

    private List<MesShiftProdCenterRecord> getUnnormalStopCenterData(List<MesShiftProdCenterRecord> shiftProdCenterRecordList, String startTime) {
        return CollectionUtils.isEmpty(shiftProdCenterRecordList) ? null :
                shiftProdCenterRecordList.stream().filter(o -> (null != o && MesExtEnumUtil.SHIFT_PROD_TYPE.UNNORMAL_STOP_CENTER.getValue() == o.getProdType() && o.getStartTime().compareTo(startTime) <= 0)).collect(Collectors.toList());
    }

    private List<MesFourMDetail> getFourMChangeData(MesWorkCenterExt workCenterExtDb, String startTime) {
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(workCenterExtDb.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(workCenterExtDb.getWorkCenterCode(), MesExtConstWords.WORK_CENTER_CODE, packBean);
        DdlPreparedPack.getStringRightLikerPack(startTime.substring(0, 10), MesExtConstWords.CREATE_DATE_TIME, packBean);
        DdlPreparedPack.getOrderByPack(new Object[]{CommonEnumUtil.ASC_OR_DESC.DESC.getValue()}, new String[]{MesExtConstWords.CREATE_DATE_TIME}, packBean);
        packBean.setHqlAppend(" limit 3 ");
        List<MesFourMDetail> fourMDetailList = fourMDetailRepository.findByHqlWhere(packBean);
        fourMDetailList.forEach(a -> {
            a.setManageTypeName(MesExtEnumUtil.MANAGE_TYPE.valueOfDescription(a.getManageType()));
        });
        return fourMDetailList;
    }

    private String getWorkCenterNotificationBarData(MesWorkCenterExt workCenterExtDb) {
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(workCenterExtDb.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(workCenterExtDb.getWorkCenterCode(), MesExtConstWords.WORK_CENTER_CODE, packBean);
        DdlPreparedPack.getOrderByPack(new Object[]{CommonEnumUtil.ASC_OR_DESC.DESC.getValue()}, new String[]{MesExtConstWords.MODIFY_DATE_TIME}, packBean);
        MesWorkCenterNotificationBar workCenterNotificationBar = workCenterNotificationBarRepository.getByProperty(packBean);
        return null == workCenterNotificationBar ? "暂无公告" : workCenterNotificationBar.getContent();
    }

    enum SHIFT_PROD_TYPE_COLOR {
        //System.out.println(String.format("#%02x%02x%02x", Color.white.getRed(), Color.white.getGreen(), Color.white.getBlue()));
        NO_WORKING(0, "无生产", "无生产", "#808080", "gray"),
        NORMAL_STOP_CENTER(10, "常规停机", "常规停机", "#808080", "gray"),
        UNNORMAL_STOP_CENTER(20, "非常规停机", "异常停机", "#ff0000", "red"),
        NORMAL_PROD(30, "常规生产", "正常生产", "#00ff00", "green"),
        UNNORMAL_PROD(40, "非常规生产", "加班生产", "#ff0000", "red");

        private int value;
        private String description;
        private String workingStatusName;
        private String color;
        private String colorName;

        SHIFT_PROD_TYPE_COLOR(int value, String description, String workingStatusName, String color, String colorName) {
            this.value = value;
            this.description = description;
            this.workingStatusName = workingStatusName;
            this.color = color;
            this.colorName = colorName;
        }

        public int getValue() {
            return value;
        }

        public static SHIFT_PROD_TYPE_COLOR getByValue(int value) {
            for (SHIFT_PROD_TYPE_COLOR matchType : values()) {
                if (matchType.getValue() == value) {
                    return matchType;
                }
            }
            return null;
        }
    }

}
